function GrainIikoSection(params)
{
	this.selector = params.selector;
	this.block = document.querySelector(this.selector);
	this.ajaxUrl = '/bitrix/components/grain/iiko.section/ajax.php';
	this.siteId = params.siteId;
	this.restaurant = params.restaurant;
	this.productData = params.productData;
	window.giBasket = params.basket;
	this.block.addEventListener('click', function(e) {
		var element=e.target.closest('[data-gi-cart-more]');
		if (!element)
			element=e.target.closest('[data-gi-cart-q]');		
		if (!element)
			return;
		e.preventDefault();
		let productHash = element.dataset.hash;
		let quantity = 1;
		this.changeCart(productHash,'more',quantity);
	}.bind(this));	
	this.block.addEventListener('click', function(e) {
		const element=e.target.closest('[data-gi-cart-less]');
		if (!element) 
			return;
		e.preventDefault();
		let productHash = element.dataset.hash;
		let quantity = 1;
		this.changeCart(productHash,'less',quantity);
	}.bind(this));
	this.initModifierSelect();	
}

GrainIikoSection.prototype.changeCart = function(productHash,action,quantity) {
	let request = new XMLHttpRequest();
	const params = "hash=" + productHash + "&restaurant=" + this.restaurant.XML_ID + "&action=" + action + "&siteId=" + this.siteId + "&quantity="+quantity;
	request.responseType = "json";
	request.open("POST", this.ajaxUrl, true);
	request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
	request.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
	request.addEventListener("readystatechange", function() {
		if(request.readyState !== 4)
			return;
		if (request.status === 200) {
			let data = request.response;			
			if(data.errors.length) {
				alert(data.errors.join("\n"));
			} else {
				if(typeof data.basketLine !== 'undefined') {
					document.querySelectorAll('[data-grain-bs-basket-line]').forEach(function(cartLine) {
						let div = document.createElement("DIV");
						div.innerHTML = data.basketLine;
						cartLine.parentNode.replaceChild(div.firstChild, cartLine);
					});
				}
				if(data.quantity>0) {
					window.giBasket[data.hash] = data.quantity;
				} else {
					if(typeof window.giBasket[data.hash] !== 'undefined')
						delete window.giBasket[data.hash];
				}
				this.setQuantity(data.hash);				
			}
		} else {
			alert('ajax request error');
		}
	}.bind(this));
	request.send(params);
};

GrainIikoSection.prototype.initModifierSelect = function() {
	this.block.addEventListener('click', this.btnNumberChange.bind(this));
	this.block.addEventListener('change', this.sizeChange.bind(this));
	this.block.addEventListener('change', this.modifierChange.bind(this));	
};

GrainIikoSection.prototype.sizeChange = function(e) {
	let sizeBlock = e.target.closest('[data-gi-s-size-block]');
	if (!sizeBlock) 
		return;
	this.recountProduct(sizeBlock.closest('[data-product]'),null);
};

GrainIikoSection.prototype.modifierChange = function(e) {
	let input = e.target.closest('[data-gi-s-modifier-quantity]');
	if(!input)
		return;
	let modifierBlock = input.closest('[data-gi-s-modifier-block]');
	if (!modifierBlock) 
		return;
	this.recountProduct(modifierBlock.closest('[data-product]'),input.dataset.giSModifierQuantity);
};

GrainIikoSection.prototype.recountProduct = function(productBlock,currentModifierId) {
	if(!productBlock || !productBlock.dataset.product)
		return;
	let productId = productBlock.dataset.product;
	let sizeBlock = productBlock.querySelector('[data-gi-s-size-block]');
	let modifierBlock = productBlock.querySelector('[data-gi-s-modifier-block]');
	let price = this.productData.prices['p'+productId];
	let discountPercent = price.price>0?(price.price - price.discountPrice)/(price.price/100):0; // only plain percent discounts works properly here!
	let sizes = typeof this.productData.sizes['p'+productId] !== 'undefined'?this.productData.sizes['p'+productId]:[];
	let modifiers = typeof this.productData.modifiers['p'+productId] !== 'undefined'?this.productData.modifiers['p'+productId]:[];
	var newPrice = Object.assign({}, price);
	var weight = this.productData.weights['p'+productId];
	var sizeSelected = null;
	productBlock.querySelectorAll(':scope input[data-gi-s-size]:checked').forEach(function(input){
		sizeSelected = input.dataset.giSSize;
		size = sizes[sizeSelected];
		newPrice.price = size.price;
	});
	var modifiersSelected = [];
	productBlock.querySelectorAll(':scope [data-gi-s-modifier-quantity]').forEach(function(input){
		if(input.dataset.giSModifierQuantity=='') { // is select, radio or checkbox
			if((input.getAttribute('type')==='radio' || input.getAttribute('type')==='checkbox') && !input.checked)
				return; 
			let modifierId = input.value;
			if(!modifierId)
				return;
			modifiersSelected[modifierId] = 1;			
		} else { // is input
			let quantity = parseInt(input.value);
			if(isNaN(quantity) || quantity<=0)
				return;
			modifierId = input.dataset.giSModifierQuantity;
			modifiersSelected[modifierId] = quantity;
		}
	});
	// apply group max/min 
	if(currentModifierId) {
		var changeModifiersQuantity = [];
		currentModifier = modifiers[currentModifierId];		
		if(typeof currentModifier['groupId'] !== 'undefined' && currentModifier['groupId'].length) {
			var currentQuantityInGroup = 0;
			for(var modifierId in modifiers)
			{
				let modifier = modifiers[modifierId];
				if(modifier['groupId']!=currentModifier['groupId'])
					continue;
				let currentQuantity = typeof modifiersSelected[modifierId]!=='undefined'?modifiersSelected[modifierId]:0;
				currentQuantityInGroup += currentQuantity;
			}
			
			var needToIncrease = !!currentModifier['groupMinAmount'] && currentQuantityInGroup<currentModifier['groupMinAmount']?currentModifier['groupMinAmount']-currentQuantityInGroup:0;
			var needToDecrease = !!currentModifier['groupMaxAmount'] && currentQuantityInGroup>currentModifier['groupMaxAmount']?currentQuantityInGroup-currentModifier['groupMaxAmount']:0;
			
			for(var modifierId in modifiers)
			{
				let modifier = modifiers[modifierId];
				if(modifier['groupId']!=currentModifier['groupId'] || modifierId==currentModifierId)
					continue;
				let currentQuantity = typeof modifiersSelected[modifierId]!=='undefined'?modifiersSelected[modifierId]:0;
				if(needToIncrease>0)
				{
					canIncrease = !!modifier['maxAmount']?modifier['maxAmount']-currentQuantity:needToIncrease;
					if(canIncrease>=needToIncrease)
					{
						changeModifiersQuantity[modifierId] = currentQuantity+needToIncrease;
						needToIncrease = 0;
					}
					else
					{
						changeModifiersQuantity[modifierId] = currentQuantity+canIncrease;
						needToIncrease -= canIncrease;
					}					
				}
				else if(needToDecrease>0)
				{
					canDescrease = !!modifier['minAmount']?currentQuantity-modifier['minAmount']:needToDecrease;
					if(canDescrease>currentQuantity)
						canDescrease=currentQuantity;
					if(canDescrease>=needToDecrease)
					{
						changeModifiersQuantity[modifierId] = currentQuantity-needToDecrease;
						needToDecrease = 0;
					}
					else
					{
						changeModifiersQuantity[modifierId] = currentQuantity-canDescrease;
						needToDecrease -= canDescrease;
					}
				}
				else
					break;
			}
			for(var modifierId in changeModifiersQuantity) {
				let quantity = changeModifiersQuantity[modifierId];
				if(quantity<=0 && typeof modifiersSelected[modifierId]!=='undefined')
					delete modifiersSelected[modifierId];
				else if(quantity>0)
					modifiersSelected[modifierId] = quantity;
				productBlock.querySelectorAll(':scope input[data-gi-s-modifier-quantity="'+modifierId+'"]').forEach(function(input){
					input.value = quantity;
				});
			}
		}
	} 
	// /apply group max/min 
	// apply modifiers price and weight
	for(var modifierId in modifiersSelected) {
		let quantity = modifiersSelected[modifierId];
		let modifier = modifiers[modifierId];
		newPrice.price += modifier.price*quantity;
		weight += modifier.weight*quantity;
	}
	// apply modifiers price and weight
	newPrice.discountPrice = newPrice.price - newPrice.price/100*discountPercent;	
	// encode product hash
	var hash = 'p'+productId;
	if(sizeSelected!==null)
		hash += '--s'+sizeSelected;
	for(var modifierId in modifiersSelected) {
		let quantity = modifiersSelected[modifierId];
		hash += '--m'+modifierId+':'+quantity;
	}
	// encode product hash	
	this.setPrice(productBlock,newPrice);
	this.setWeight(productBlock,weight);
	this.setButtonsHash(productBlock,hash);
	this.setQuantity(hash);
};

GrainIikoSection.prototype.setButtonsHash = function(productBlock,hash) {
	productBlock.querySelectorAll(':scope [data-gi-cart-q], :scope [data-gi-cart-more], :scope [data-gi-cart-less]').forEach(function(button){
		button.dataset.hash = hash;
	});
};

GrainIikoSection.prototype.setQuantity = function(hash) {
	let quantity = typeof window.giBasket[hash] !== 'undefined'?window.giBasket[hash]:0;
	if(quantity>0) {
		document.querySelectorAll('[data-gi-cart-more][data-hash="'+hash+'"]').forEach(function(button) {
			button.innerHTML = button.dataset.moreMess;
		});
		document.querySelectorAll('[data-gi-cart-q][data-hash="'+hash+'"]').forEach(function(button) {
			button.innerHTML = quantity + (button.dataset.measure?' '+button.dataset.measure:'');
			button.style.display = '';
		});
		document.querySelectorAll('[data-gi-cart-less][data-hash="'+hash+'"]').forEach(function(button) {
			button.style.display = '';
		});
	} else {
		document.querySelectorAll('[data-gi-cart-more][data-hash="'+hash+'"]').forEach(function(button) {
			button.innerHTML = button.dataset.addMess;
		});
		document.querySelectorAll('[data-gi-cart-q][data-hash="'+hash+'"]').forEach(function(button) {
			button.style.display = 'none';
		});
		document.querySelectorAll('[data-gi-cart-less][data-hash="'+hash+'"]').forEach(function(button) {
			button.style.display = 'none';
		});
	}
};

GrainIikoSection.prototype.setPrice = function(productBlock,newPrice) {
	productBlock.querySelectorAll('[data-price]').forEach(function(priceNode){
		priceNode.innerHTML = this.currencyFormat(newPrice.discountPrice,newPrice.currency);
	}.bind(this));
	productBlock.querySelectorAll('[data-base-price]').forEach(function(priceNode){
		priceNode.innerHTML = this.currencyFormat(newPrice.price,newPrice.currency);
		priceNode.style.display = newPrice.discountPrice<newPrice.price?'':'none';
	}.bind(this));	
};

GrainIikoSection.prototype.setWeight = function(productBlock,weight) {
	productBlock.querySelectorAll('[data-weight]').forEach(function(weightNode){
		weightNode.innerHTML = this.number_format(weight*1000,0,'.',' ');
	}.bind(this));
};

GrainIikoSection.prototype.btnNumberChange = function(e) {
	let button = e.target.closest('[data-gi-s-number]');
	if (!button) 
		return;
	let input = button.parentNode.querySelector(':scope input[type=number]');
	if (!input) 
		return;
	e.preventDefault();
	if(button.dataset.giSNumber=='down')
		input.stepDown();
	else
		input.stepUp();
	if(input.dataset)
	this.recountProduct(input.closest('[data-product]'),input.dataset.giSModifierQuantity);
	//input.dispatchEvent(new Event('change'));
};

GrainIikoSection.prototype.currencyFormat = function(price, currency) {
	var result = '';
	const format = this.productData.currencies[currency];
	if (typeof format !== 'undefined') {
		format.CURRENT_DECIMALS = format.DECIMALS;
		if (format.HIDE_ZERO === 'Y' && price == parseInt(price, 10))
			format.CURRENT_DECIMALS = 0;
		result = this.number_format(price, format.CURRENT_DECIMALS, format.DEC_POINT, format.THOUSANDS_SEP);
			result = format.FORMAT_STRING.replace(/(^|[^&])#/, '$1' + result);
	} else 
		result = price;
	return result;
};

GrainIikoSection.prototype.number_format = function(number, decimals, dec_point, thousands_sep) // BX.util.number_format clone
{
	var i, j, kw, kd, km, sign = '';
	decimals = Math.abs(decimals);
	if (isNaN(decimals) || decimals < 0)
		decimals = 2;
	dec_point = dec_point || ',';
	if (typeof thousands_sep === 'undefined' || thousands_sep===null)
		thousands_sep = '';
	number = (+number || 0).toFixed(decimals);
	if (number < 0) {
		sign = '-';
		number = -number;
	}
	i = parseInt(number, 10) + '';
	j = (i.length > 3 ? i.length % 3 : 0);

	km = (j ? i.substr(0, j) + thousands_sep : '');
	kw = i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousands_sep);
	kd = (decimals ? dec_point + Math.abs(number - i).toFixed(decimals).replace(/-/, '0').slice(2) : '');
	return sign + km + kw + kd;
};
